@@ -2,6 +2,7 @@
     Float2Int.h
 
     Copyright 2010, Alan Calvert
+    Copyright 2018, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -15,6 +16,8 @@
 
     You should have received a copy of the GNU General Public License
     along with yoshimi.  If not, see <http://www.gnu.org/licenses/>.
+
+    Modified March 2018
 */
 
 #ifndef FLOAT2INT_H
@@ -30,10 +33,12 @@ class Float2Int {
         ~Float2Int() { }
 
     protected:
-        const int float2int(const float val) const;
+        // const int float2int(const float val) const; // gives compiler warning :(
+        int float2int(const float val) const;
 };
 
-inline const int Float2Int::float2int(const float val) const
+//inline const int Float2Int::float2int(const float val) const
+inline int Float2Int::float2int(const float val) const
     { return (int)((isgreater(val, 0.0f)) ? (int)truncf(val) : (int)truncf((val - 1.0f))); }
     // for rationale, see <http://www.mega-nerd.com/FPcast/>
 
