@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2017, Will Godfrey
+    Copyright 2014-2018, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
     This file is derivative of original ZynAddSubFX code.
 
-    Modified December 2017
+    Modified April 2018
 */
 
 #include <zlib.h>
@@ -63,11 +63,17 @@ XMLwrapper::XMLwrapper(SynthEngine *_synth, bool _isYoshi) :
     memset(&parentstack, 0, sizeof(parentstack));
     tree = mxmlNewElement(MXML_NO_PARENT, "?xml version=\"1.0\" encoding=\"UTF-8\"?");
     mxml_node_t *doctype = mxmlNewElement(tree, "!DOCTYPE");
-    bool yoshiType = (synth->getRuntime().xmlType <= XML_PRESETS);
-    // cout << "yoshiType  " << yoshiType << "   isYoshi " << isYoshi << endl;
 
-    if (!yoshiType || !isYoshi)
+    if (isYoshi)
     {
+        //cout << "Our doctype" << endl;
+        mxmlElementSetAttr(doctype, "Yoshimi-data", NULL);
+        root = mxmlNewElement(tree, "Yoshimi-data");
+        information.yoshiType = 1;
+    }
+    else
+    {
+        //cout << "Zyn doctype" << endl;
         mxmlElementSetAttr(doctype, "ZynAddSubFX-data", NULL);
         root = mxmlNewElement(tree, "ZynAddSubFX-data");
         mxmlElementSetAttr(root, "version-major", "3");
@@ -75,12 +81,7 @@ XMLwrapper::XMLwrapper(SynthEngine *_synth, bool _isYoshi) :
         mxmlElementSetAttr(root, "ZynAddSubFX-author", "Nasca Octavian Paul");
         information.yoshiType = 0;
     }
-    else
-    {
-        mxmlElementSetAttr(doctype, "Yoshimi-data", NULL);
-        root = mxmlNewElement(tree, "Yoshimi-data");
-        information.yoshiType = 1;
-    }
+
     node = root;
     mxmlElementSetAttr(root, "Yoshimi-author", "Alan Ernest Calvert");
     string tmp = YOSHIMI_VERSION;
@@ -114,6 +115,8 @@ XMLwrapper::XMLwrapper(SynthEngine *_synth, bool _isYoshi) :
                 addparbool("enable_gui", synth->getRuntime().showGui);
                 addparbool("enable_splash", synth->getRuntime().showSplash);
                 addparbool("enable_CLI", synth->getRuntime().showCLI);
+                addparbool("enable_auto_instance", synth->getRuntime().autoInstance);
+                addparU("active_instances", synth->getRuntime().activeInstance);
             endbranch();
         }
     }
@@ -127,7 +130,7 @@ XMLwrapper::~XMLwrapper()
 }
 
 
-bool XMLwrapper::checkfileinformation(const string& filename)
+void XMLwrapper::checkfileinformation(const string& filename)
 {
     stackpos = 0;
     memset(&parentstack, 0, sizeof(parentstack));
@@ -137,24 +140,22 @@ bool XMLwrapper::checkfileinformation(const string& filename)
     tree = NULL;
     char *xmldata = doloadfile(filename);
     if (!xmldata)
-        return -1;
+        return;
 
     char *first = strstr(xmldata, "<!DOCTYPE Yoshimi-data>");
     information.yoshiType = (first!= NULL);
-    bool bRet = false; // we're not actually using this!
     char *start = strstr(xmldata, "<INFORMATION>");
     char *end = strstr(xmldata, "</INFORMATION>");
     if (!start || !end || start >= end)
     {
-        bRet = slowinfosearch(xmldata);
+        slowinfosearch(xmldata);
         delete [] xmldata;
-        return bRet;
+        return;
     }
 
     // Andrew: just make it simple
     // Will: but not too simple :)
     char *idx = start;
-    *end = 0; // fiddle to limit search - I can't find a better way :(
     unsigned short names = 0;
 
     /* the following could be in any order. We are checking for
@@ -184,31 +185,22 @@ bool XMLwrapper::checkfileinformation(const string& filename)
             information.PADsynth_used = 1;
     }
 
-    if (names == 7)
-    {
-        bRet = true;
-    }
-    else
-    {
-//        if (strstr(idx, "<INSTRUMENT_KIT>"))
-//            synth->getRuntime().Log("Oops! Shouldn't find it.");
-        *end = 0x3c; // restore full length
-        bRet = slowinfosearch(xmldata);
-    }
+    if (names != 7)
+        slowinfosearch(xmldata);
+
     delete [] xmldata;
-    return bRet;
+    return;
 }
 
 
-bool XMLwrapper::slowinfosearch(char *idx)
+void XMLwrapper::slowinfosearch(char *idx)
 {
     idx = strstr(idx, "<INSTRUMENT_KIT>");
     if (idx == NULL)
-        return false;
+        return;
 
     string mark;
     int max = NUM_KIT_ITEMS;
-
     /*
      * The following *must* exist, otherwise the file is corrupted.
      * They will always be in this order, which means we only need
@@ -218,7 +210,7 @@ bool XMLwrapper::slowinfosearch(char *idx)
      */
     idx = strstr(idx, "name=\"kit_mode\"");
     if (idx == NULL)
-        return false;
+        return;
     if (strncmp(idx + 16 , "value=\"0\"", 9) == 0)
         max = 1;
 
@@ -227,11 +219,11 @@ bool XMLwrapper::slowinfosearch(char *idx)
         mark = "<INSTRUMENT_KIT_ITEM id=\"" + asString(kitnum) + "\">";
         idx = strstr(idx, mark.c_str());
         if (idx == NULL)
-            return false;
+            return;
 
         idx = strstr(idx, "name=\"enabled\"");
         if (idx == NULL)
-            return false;
+            return;
         if (!strstr(idx, "name=\"enabled\" value=\"yes\""))
             continue;
 
@@ -239,7 +231,7 @@ bool XMLwrapper::slowinfosearch(char *idx)
         {
             idx = strstr(idx, "name=\"add_enabled\"");
             if (idx == NULL)
-                return false;
+                return;
             if (strncmp(idx + 26 , "yes", 3) == 0)
                 information.ADDsynth_used = 1;
         }
@@ -247,7 +239,7 @@ bool XMLwrapper::slowinfosearch(char *idx)
         {
             idx = strstr(idx, "name=\"sub_enabled\"");
             if (idx == NULL)
-                return false;
+                return;
             if (strncmp(idx + 26 , "yes", 3) == 0)
                 information.SUBsynth_used = 1;
         }
@@ -255,7 +247,7 @@ bool XMLwrapper::slowinfosearch(char *idx)
         {
             idx = strstr(idx, "name=\"pad_enabled\"");
             if (idx == NULL)
-                return false;
+                return;
             if (strncmp(idx + 26 , "yes", 3) == 0)
                 information.PADsynth_used = 1;
         }
@@ -263,11 +255,10 @@ bool XMLwrapper::slowinfosearch(char *idx)
           & information.SUBsynth_used
           & information.PADsynth_used)
         {
-//            synth->getRuntime().Log("Kit count " + asString(kitnum));
             break;
         }
     }
-  return true;
+  return;
 }
 
 
@@ -381,6 +372,12 @@ char *XMLwrapper::getXMLdata()
 }
 
 
+void XMLwrapper::addparU(const string& name, unsigned int val)
+{
+    addparams2("parU", "name", name.c_str(), "value", asString(val));
+}
+
+
 void XMLwrapper::addpar(const string& name, int val)
 {
     addparams2("par", "name", name.c_str(), "value", asString(val));
@@ -624,6 +621,23 @@ int XMLwrapper::getbranchid(int min, int max)
 }
 
 
+unsigned int XMLwrapper::getparU(const string& name, unsigned int defaultpar, unsigned int min, unsigned int max)
+{
+    node = mxmlFindElement(peek(), peek(), "parU", "name", name.c_str(), MXML_DESCEND_FIRST);
+    if (!node)
+        return defaultpar;
+    const char *strval = mxmlElementGetAttr(node, "value");
+    if (!strval)
+        return defaultpar;
+    unsigned int val = string2int(strval);
+    if (val < min)
+        val = min;
+    else if (val > max)
+        val = max;
+    return val;
+}
+
+
 int XMLwrapper::getpar(const string& name, int defaultpar, int min, int max)
 {
     node = mxmlFindElement(peek(), peek(), "par", "name", name.c_str(), MXML_DESCEND_FIRST);
@@ -664,6 +678,7 @@ int XMLwrapper::getparbool(const string& name, int defaultpar)
     char tmp = strval[0] | 0x20;
     return (tmp != '0' && tmp != 'n' && tmp != 'f') ? 1 : 0;
 }
+// case insensitive, anything other than '0', 'no', 'false' is treated as 'true'
 
 
 string XMLwrapper::getparstr(const string& name)
